-- Init
local frame = CreateFrame("FRAME", "TurnInEverything_Frame");
frame:RegisterEvent("GOSSIP_SHOW");
frame:RegisterEvent("QUEST_GREETING");
frame:RegisterEvent("QUEST_PROGRESS");
frame:RegisterEvent("QUEST_COMPLETE");
frame:RegisterEvent("MODIFIER_STATE_CHANGED");
local tie_TempDisable = false;
local tie_SkipNumGossips = 0;
local tie_LastGossipTime = 0;
local tie_DEBUG = false;

-- Event handler
local function tie_EventHandler(self, event)
	if (tie_DEBUG) then
		print('tie_eventHandler:', event);
		print('tie_SkipNumGossips:', tie_SkipNumGossips);
		print('tie_LastGossipTime:', tie_LastGossipTime);
	end

	-- Track modifier changes
	if (event == "MODIFIER_STATE_CHANGED") then
		tie_ModiferChange();
		return;
	
	-- Track quest progress
	elseif (event == "QUEST_PROGRESS" or event == "QUEST_COMPLETE") then
		tie_QuestProgress(event);
		return;

	-- Track new gossip frames
	elseif (event == "GOSSIP_SHOW") then
		-- Do not process GOSSIP_SHOW if tie_TempDisable is set
		if (tie_TempDisable) then
			return;
		end
		
		-- Do not process GOSSIP_SHOW if tie_SkipNumGossips > 0
		if (tie_SkipNumGossips > 0) then
			-- Ignore tie_SkipNumGossips if it's been more than a few of seconds (in case GOSSIP_SHOW only fired once)
			if (tie_LastGossipTime + 3 < GetTime()) then
				if (tie_DEBUG) then print('tie_EventHandler: Ignoring tie_SkipNextGossip due to age'); end
				tie_SkipNumGossips = 0;

			-- Otherwise decrement tie_SkipNumGossips and skip this round
			else
				if (tie_DEBUG) then print('tie_EventHandler: Exiting on tie_SkipNextGossip'); end
				tie_SkipNumGossips = tie_SkipNumGossips - 1;
				return;
			end
		end
		
		-- Process each gossip option
		tie_GossipShow();
		return;
		
	-- Public debug for QUEST_GREETING
	elseif (event == "QUEST_GREETING") then
		print('TurnInEverything: This NPC uses the QUEST_GREETING frame. Please report this NPC name to the author -- zach-curse@kotlarek.com');
		return;
	end
end
frame:SetScript('OnEvent', tie_EventHandler);

-- Check each active quest
function tie_GossipShow()
	-- Record the processing time
	tie_LastGossipTime = GetTime();

	-- Find all active quests
	local q = { GetGossipActiveQuests() };
	local i = 1;
	local qNum = 1;

	-- Loop through the quest details
	-- We need this i+3 complication to avoid the nils in q
	while(q[i]) do
		if (tie_DEBUG) then print('tie_GossipShow: Processing quest:', qNum); end
		
		-- Select the quest and let it throw QUEST_PROGRESS
		SelectGossipActiveQuest(qNum);
		i = i + 3;
		qNum = qNum + 1;

		-- Skip two GOSSIP_SHOW events, as they are often doubled
		tie_SkipNumGossips = 2;
	end
end

-- If this quest is completable, click continue and spew out text until we get to the completion screen
function tie_QuestProgress(event)
	-- If we're done with the progress text
	if (event == "QUEST_COMPLETE") then
		-- Display the reward text
		DEFAULT_CHAT_FRAME:AddMessage(GetTitleText() .. ' Reward: ' .. GetRewardText(), 1, 0.98, 0.8);
	
		-- Do not accept the reward if tie_TempDisable is set
		if (tie_TempDisable) then
			return;
		end
		
		-- If there's only one reward, grab it
		if (GetNumQuestChoices() < 2) then
			GetQuestReward(1);
		end

	-- If there's still progress text left
	else
		-- Display the progress text
		DEFAULT_CHAT_FRAME:AddMessage(GetTitleText() .. ' Progress: ' .. GetProgressText(), 1, 0.98, 0.8);

		-- Bail if the quest is not completable
		if(not IsQuestCompletable()) then
			DeclineQuest();
			tie_IgnoreNext = true;
			return;
		end

		-- Do not complete the quest if tie_TempDisable is set
		if (tie_TempDisable) then
			return;
		end

		-- Otherwise click continue continue
		CompleteQuest();
	end
end

-- Temporarily disable processing if the hotkey is pressed
-- Currently hard-coded to shift, but abstracted to a function should I ever to to writing a UI to allow user configuration
function tie_ModiferChange()
	if (IsShiftKeyDown()) then
		tie_TempDisable = true;
	else
		tie_TempDisable = false;
	end
end